@@ -8,6 +8,7 @@ import matplotlib.pyplot as plt
 import pandas as pd
 from PIL import Image
 from .dtr import *
+from .utils import *
 plt.rcParams['font.size'] = 10 #font size
 plt.rcParams['figure.figsize'] = 20,70 #figure size
 
@@ -33,6 +34,7 @@ class CBIR:
 
     def create_db(self,
                     df_attr: Any,
+                    dtrs: Union[None, np.ndarray] = None,
                     img_attr: str = "imgfile",
                     case_attr: str = "patient",
                     type_attr: str = "tissue",
@@ -42,6 +44,7 @@ class CBIR:
 
         Args:
             df_attr (Any): Pandas dataframe containing at least image files and case IDs.
+            dtrs (np.ndarray, optional): pre-computed DTRs for the image files. The order should be the same as the img_attr. Default to None. 
             img_attr (str, optional): Column name of image files in df_attr. Defaults to "imgfile".
             case_attr (str, optional): Column name of case ID in df_attr. Defaults to "patient".
             type_attr (str, optional): Column name of additional attribute to show in df_attr. Defaults to "tissue".
@@ -55,7 +58,10 @@ class CBIR:
 
         imgfiles = df_attr[img_attr]
 
-        self.dtrs = self.dtr_obj.get_dtr_multifiles(imgfiles)
+        if dtrs is None:
+            self.dtrs = self.dtr_obj.get_dtr_multifiles(imgfiles)
+        else:
+            self.dtrs = dtrs
             
         #make index    
         params = {'M': 20, 'post': 0, 'efConstruction': 500}
@@ -127,7 +133,7 @@ class CBIR:
         labels = ["{}\n{}\n{}".format(os.path.basename(d[1][self.img_attr]), 
                                       d[1][self.type_attr], 
                                       d[1][self.case_attr]) for d in df_tmp.iterrows()]
-        self._imgcats(df_tmp[self.img_attr], labels=labels)
+        imgcats(df_tmp[self.img_attr], labels=labels)
             
 
     def search(self,
@@ -138,6 +144,9 @@ class CBIR:
                scale: Union[None, int] = None,
                fkey: Union[None, str] = None,
                fval: Union[str, List[str], None] = None,
+               dpi: int = 320, 
+               save: bool = False, 
+               outfile: str = "", 
                ) -> Tuple[np.ndarray, pd.DataFrame]:
         """Search and show images similar to the query image using DTR
 
@@ -149,15 +158,21 @@ class CBIR:
             scale (Union[None, int], optional): Query image is rescaled. Default to None.
             fkey (Union[None, str]): Key for filter in df_attr. Default to None.
             fval (Union[str, List[str], None]): Value(s) for filter. Default to None.
+            dpi (int, optional): Dots per inch (DPI) of output image. Defaults to 320.
+            save (bool, optional): Save the output image to outfile if True. Defaults to False.
+            outfile (str, optional): Output image file. Defaults to "".
         Returns:
             Tuple[np.ndarray, pd.DataFrame]: Retrieved image file and the corresponding info(case, similarity, and attribute).
         """
-        if type(filter_val) == str:
-            filter_val = [filter_val]
+        if type(fval) == str:
+            fval = [fval]
 
         if fkey is not None:
             if not fkey in self.df_attr.columns:
                 raise Exception("invalid key for filter {}".format(fkey))
+
+        if save is False:
+            outfile = ""
         
 
         qdtr = self.dtr_obj.get_dtr(qimgfile, scale=scale)
@@ -200,9 +215,9 @@ class CBIR:
         labels = ["{}\n{}\n{:.3f}".format(attr, patient, s) for attr,patient,s in zip(attrs, patients, sims)]
         if show:
             if show_query:
-                self._imgcats([qimgfile, *imgfiles], labels=["query", *labels])
+                imgcats([qimgfile, *imgfiles], labels=["query", *labels], save=outfile, dpi=dpi)
             else:
-                self._imgcats(imgfiles, labels=labels)
+                imgcats(imgfiles, labels=labels, save=outfile, dpi=dpi)
 
         return imgfiles, pd.DataFrame({'attr':attrs, 'case':patients, 'similarity':sims})
 
@@ -231,6 +246,9 @@ class CBIR:
                     scale: Union[None, int] = None,
                     fkey: Union[None, str] = None,
                     fval: Union[str, List[str], None] = None,
+                    dpi: int = 320,
+                    save: bool = False,
+                    outfile: str = "", 
                     ) -> pd.DataFrame:
         """Search and show images similar to the query image using DTR
 
@@ -243,16 +261,22 @@ class CBIR:
             scale (Union[None, int], optional): Query images are rescaled. Default to None.
             fkey (Union[None, str]): Key for filter in df_attr. Default to None.
             fval (Union[str, List[str], None]): Value(s) for filter. Default to None.
+            dpi (int, optional): Dots per inch (DPI) of output image. Defaults to 320.
+            save (bool, optional): Save the output image to outfile if True. Defaults to False.
+            outfile (str, optional): Output image file. Defaults to "".
         Returns:
             pd.DataFrame : Results
         """
-        if type(filter_val) == str:
-            filter_val = [filter_val]
+        if type(fval) == str:
+            fval = [fval]
 
         if fkey is not None:
             if not fkey in self.df_attr.columns:
                 raise Exception("invalid key for filter {}".format(fkey))
         
+        if save is False:
+            outfile = ""
+        
         if not strategy in ['max', 'mean']:
             raise Exception(f'invalid strategy: {strategy}')
 
@@ -316,26 +340,13 @@ class CBIR:
                 for j in range(qn):
                     plt.subplot(ncols, nrows, j + i * qn + offset)
                     imgfile = d[1][f'imgfile_{j}']
-                    if os.path.exists(imgfile):
-                        im_list = np.asarray(Image.open(imgfile)) 
-                        plt.imshow(im_list)
-                        plt.axis('off')
+                    if type(imgfile) != int:
+                        if os.path.exists(imgfile):
+                            im_list = np.asarray(Image.open(imgfile)) 
+                            plt.imshow(im_list)
+                            plt.title('sim:{}'.format(d[1][f'sim_{j}']))
+                    plt.axis('off')
+            if save is True:
+                plt.savefig(outfile, dpi=dpi)
             
         return df_merged
-
-    def _imgcats(self, 
-                    infiles: List[str], 
-                    labels: List[str], 
-                    nrows: int = 3, 
-                    ) -> None:
-
-        ncols = int(np.ceil(len(infiles)/nrows))
-        for i, infile in enumerate(infiles):
-            plt.subplot(ncols, nrows, i+1)
-            im = Image.open(infile)
-            im_list = np.asarray(im)
-            plt.imshow(im_list)
-            if len(labels) != 0:
-                plt.title(labels[i])
-            plt.axis('off')
-        plt.show()
\ No newline at end of file
