@@ -1,7 +1,9 @@
 from typing import Any, List, Tuple, Union
 from PIL import Image
+from pyrsistent import mutant
 import numpy as np
 import cv2
+import pandas as pd
 import tensorflow as tf
 from tensorflow.keras import models, preprocessing
 from tensorflow.keras.applications import resnet50, vgg16, mobilenet_v2, inception_v3, nasnet, densenet, inception_resnet_v2
@@ -104,6 +106,7 @@ class DTR:
                 angle: Union[None, int, List[int]] = None,
                 size: Union[None, int] = None,
                 scale: Union[None, float] = None,
+                multi_scale: bool = False,
                 ) -> np.ndarray:
         """Calculates DTR for an image object or file.
 
@@ -112,6 +115,7 @@ class DTR:
             angle (Union[None, int, List[int]], optional): Rotation angle(s) (0-360). If list is given, mean DTRs of the rotated image return. Defaults to None.
             size (Union[None, int], optional): Image is resized to the given size. Default to None.
             scale (Union[None, int], optional): Image is rescaled. Active only size is not specified. Default to None.
+            multi_scale (bool, optional): DTR for 1/4 sized image is concatenated. The dimension of the DTR will be  2*dim. Default to False.
 
         Returns:
             np.ndarray: DTR for the image
@@ -130,12 +134,17 @@ class DTR:
             h, w, _ = x.shape
             x = cv2.resize(x, dsize=[int(h*scale), int(w*scale)])
 
+        if multi_scale:
+            #1/4 scale
+            x2 = cv2.resize(x, dize=None, fx=0.25, fy=0.25)
 
         if angle is not None:
             if type(angle) == int:
                 x = preprocessing.image.apply_affine_transform(x, theta = angle)
+                if multi_scale: 
+                    x2 = preprocessing.image.apply_affine_transform(x2, theta = angle)
             elif type(angle) == list:
-                dtrs = np.vstack([self.get_dtr(img, theta) for theta in angle])
+                dtrs = np.vstack([self.get_dtr(img, theta, size, scale, multi_scale) for theta in angle])
                 dtr_mean = np.mean(dtrs, axis=0)
                 return dtr_mean / np.linalg.norm(dtr_mean, ord=2) #L2-normalize
             else:
@@ -144,7 +153,15 @@ class DTR:
 
         x = np.expand_dims(x, axis=0)
         x = self.prep(x)
-        return np.array(self.cbp([x])[0])
+        dtr = np.array(self.cbp([x])[0])
+
+        if multi_scale:
+            x2 = np.expand_dims(x2, axis=0)
+            x2 = self.prep(x2)
+            dtr2 = np.array(self.cbp([x2])[0])
+            dtr = np.concatenate([dtr, dtr2])
+
+        return dtr
 
     def sim(self, 
             x: np.ndarray,
@@ -184,28 +201,38 @@ class DTR:
     
     def get_mean_dtrs(self,
                       dtrs: np.ndarray,
-                      imgfiles: List[str],
                       cases: List[str],
-                      ) -> Tuple[np.ndarray, List[str], List[str]]:
+                      df: Union[None, pd.DataFrame, List[str]] = None,
+                      ) -> Tuple[np.ndarray, List[str], Union[None, pd.DataFrame, List[str]]]:
         """Calculate mean dtrs.
 
         Args:
             dtrs (np.ndarray): M-dimensional DTRs for N images (NxM array).
-            imgfiles (List[str]): List of full image file path for N image.
             cases (List[str]): List of case IDs for N images.
+            df: Union[None, pd.DataFrame, List[str]]: List or dataframe containing attributes of N images. The order should be the same as dtrs and cases. Default to None.
 
         Returns:
             Tuple[np.ndarray, List[str], List[str]]: mean DTRs, 
             List of image file path of the representative images (medoid for each case), and case IDs.
         """
+
+        
         u_cases = np.sort(np.unique(cases))
         dtrs_mean = np.vstack([np.mean(dtrs[np.array(cases)==case, :], axis=0) for case in u_cases])
         dtrs_mean = dtrs_mean / np.linalg.norm(dtrs_mean, ord=2) #L2-normalize
 
         medoid_dict = get_medoid(dtrs, cases)
 
-        imgfiles_mean=[imgfiles[medoid_dict[case]] for case in u_cases]
-        
-        return dtrs_mean, imgfiles_mean, list(u_cases)
+        if df is not None:
+            if type(df) == list:
+                df_mean =[df[medoid_dict[case]] for case in u_cases]
+            else:
+                #df = df.reset_index()
+                idx = np.array([medoid_dict[case] for case in u_cases])
+                df_mean = df.iloc[idx, :]
+        else:
+            df_mean = None
+                
+        return dtrs_mean, list(u_cases), df_mean
         
         
