@@ -199,7 +199,7 @@ def plt_dtr_attr(X: np.ndarray,
                         color='attr',
                         size='size',
                         hover_name='attr',
-                        hover_data = cases,
+                        hover_data = ['x1','x2','attr','medoid'],
                         text=text,
                         range_x = [np.min(X_emb[:,0])-width1, np.max(X_emb[:,0])+width1],
                         range_y = [np.min(X_emb[:,1])-height1, np.max(X_emb[:,1])+height1],
@@ -209,7 +209,9 @@ def plt_dtr_attr(X: np.ndarray,
             fig.update_yaxes(visible=False)
         if save:
             print ("dpi is ignored in plotly mode.")
-            fig.write_image(outfile)
+            fig.write_html(outfile)
+        fig.show(renderer='iframe')
+        return df, fig
 
     else:
         plt.clf()
